#include <cmath>
#include "color.h"
#include "colorcurve.h"
#include "mesh.h"
#include "shader.h"
#include "texture2d.h"

using namespace std;

namespace Msp {
namespace GL {

ColorCurve::ColorCurve():
	shprog("colorcurve.glsl"),
	quad(get_fullscreen_quad())
{
	shdata.uniform("source", 0);
	shdata.uniform("curve", 1);

	curve.storage(LUMINANCE, 256);
	curve.set_min_filter(LINEAR);
	curve.set_wrap(CLAMP_TO_EDGE);

	set_peak(0.2);
	set_brightness(1.5);
	set_linear();
}

void ColorCurve::set_peak(float p)
{
	if(p<0 || p>1)
		throw invalid_argument("ColorCurve::set_peak");
	shdata.uniform("peak", p);
}

void ColorCurve::set_brightness(float b)
{
	if(b<1)
		throw invalid_argument("ColorCurve::set_brightness");
	shdata.uniform("brightness", b);
}

void ColorCurve::set_gamma(float g)
{
	if(g<0.1 || g>10)
		throw invalid_argument("ColorCurve::set_gamma");

	unsigned char curve_data[256];
	for(unsigned i=0; i<256; ++i)
		curve_data[i] = pow(i/255.0f, 1/g)*255+0.5f;
	curve.image(0, LUMINANCE, UNSIGNED_BYTE, curve_data);
}

void ColorCurve::set_srgb()
{
	unsigned char curve_data[256];
	curve_data[0] = 0;
	for(unsigned i=1; i<256; ++i)
		curve_data[i] = to_srgb(i/255.0f)*255+0.5f;
	curve.image(0, LUMINANCE, UNSIGNED_BYTE, curve_data);
}

void ColorCurve::set_linear()
{
	unsigned char curve_data[256];
	for(unsigned i=0; i<256; ++i)
		curve_data[i] = i;
	curve.image(0, LUMINANCE, UNSIGNED_BYTE, curve_data);
}

void ColorCurve::render(const Texture2D &color_buf, const Texture2D &)
{
	Bind _bind_shader(shprog);
	shdata.apply();
	Bind _bind_mesh(quad);
	Bind _bind_tex(color_buf);
	Bind _bind_curve(curve, 1);
	quad.draw();
}

} // namespace GL
} // namespace Msp
