#ifndef MSP_GL_INSTANCESCENE_H_
#define MSP_GL_INSTANCESCENE_H_

#include <map>
#include <set>
#include "scene.h"

namespace Msp {
namespace GL {

/**
A Scene optimized for rendering instanced Renderables, such as ObjectInstances.
All Renderables with the same instance key are rendered consecutively; within
the same key rendering order is unspecified.
*/
class InstanceScene: public Scene
{
private:
	typedef std::set<const Renderable *> RenderableSet;
	typedef std::map<long, RenderableSet> InstanceMap;

	InstanceMap renderables;

public:
	virtual void add(const Renderable &);
	virtual void remove(const Renderable &);

	virtual void setup_frame() const;
	virtual void finish_frame() const;

	using Scene::render;
	virtual void render(Renderer &, const Tag &tag = Tag()) const;
};

} // namespace GL
} // namespace Msp

#endif
