#ifndef MSP_GL_RENDERPASS_H_
#define MSP_GL_RENDERPASS_H_

#include <msp/core/refptr.h>
#include <msp/datafile/objectloader.h>
#include "bindable.h"

namespace Msp {
namespace GL {

class Material;
class Program;
class ProgramData;
class Renderer;
class Texture;
class Texturing;

/**
Encapsulates the data that determines the appearance of a rendered surface.
This includes shader and data for it, material and texturing.

XXX Does not delete inline texture from datafiles properly
*/
class RenderPass
{
public:
	class Loader: public DataFile::CollectionObjectLoader<RenderPass>
	{
	public:
		Loader(RenderPass &);
		Loader(RenderPass &, Collection &);

	private:
		void init();
		void material_inline();
		void material(const std::string &);
		void texunit(unsigned);
		void texunit_named(unsigned, const std::string &);
		void uniforms();
	};

private:
	struct TextureLoader: public DataFile::CollectionObjectLoader<Texturing>
	{
	private:
		unsigned index;
		RefPtr<Texture> tex;

	public:
		TextureLoader(Texturing &, unsigned, Collection *);
	private:
		virtual void finish();

		void texture(const std::string &);
		void texture2d();
	};

	const Program *shprog;
	RefPtr<const ProgramData> shdata;
	RefPtr<const Material> material;
	Texturing *texturing;
	std::map<std::string, unsigned> tex_names;
	bool back_faces;

	RenderPass &operator=(const RenderPass &);
public:
	RenderPass();
	RenderPass(const RenderPass &);
	~RenderPass();

	void set_shader_program(const Program *, const ProgramData *);
	const Program *get_shader_program() const { return shprog; }
	const ProgramData *get_shader_data() const { return shdata.get(); }
	void set_material(const Material *);
	const Material *get_material() const { return material.get(); }
	void set_texture(unsigned, const Texture *);
	const Texturing *get_texturing() const { return texturing; }
	int get_texture_index(const std::string &) const;
	void set_back_faces(bool);
	bool get_back_faces() const { return back_faces; }

	void apply(Renderer &) const;
};

} // namespace GL
} // namespace Msp

#endif
